﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ThinkGeo.MapSuite.Core;
using System.IO;

namespace WmkRenderLayer
{
    /// <summary>
    /// This class inherits from FileBitmapTileCache to attempt preventing 
    /// saving and loading of bad tiles.  See this post for more info:
    /// http://gis.thinkgeo.com/Support/DiscussionForums/tabid/143/aff/39/aft/8446/afv/topic/Default.aspx#21758
    /// </summary>
    public class ExtendedFileBitmapTileCache : FileBitmapTileCache
    {
        public ExtendedFileBitmapTileCache(string cacheDirectory)
        : base(cacheDirectory)
        { }

        public ExtendedFileBitmapTileCache(string cacheDirectory, string cacheId)
            : base(cacheDirectory, cacheId)
        { }

        protected override void SaveTileCore(ThinkGeo.MapSuite.Core.Tile tile)
        {
            BitmapTile bitmapTile = tile as BitmapTile;
            if (bitmapTile != null)
            {
                using (MemoryStream stream = new MemoryStream())
                {
                    bitmapTile.Bitmap.Save(stream, System.Drawing.Imaging.ImageFormat.Png);
                    long sizeInKB = stream.Length / 1024;
                    if (sizeInKB > 2)
                    {
                        try
                        {
                            base.SaveTileCore(tile);
                        }
                        catch (Exception ex)
                        { 

                        }
                    }
                }
            }
        }    
    }
}
