﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Runtime.Serialization;
using System.ComponentModel;
using System.Diagnostics;
using System.Reflection;
using System.Xml.Serialization;
using System.Windows.Threading;
using System.Text;
using System.Text.RegularExpressions;


namespace WmkRenderLayer
{

    [Serializable]
    [CollectionDataContract(ItemName = "s", Namespace = "ems.entities")]
    public class StringCollection : ObservableCollection<string>, INotifyPropertyChanged
    {
        [field: NonSerialized]
        public event PropertyChangedEventHandler PropertyChanged;

        public StringCollection()
        {
        }

        public StringCollection(IEnumerable<string> strings)
        {
            ReplaceStrings(strings);
        }

        public void ReplaceStrings(IEnumerable<string> strings)
        {
            if (strings != null)
            {
                this.Clear();
                foreach (var _string in strings)
                this.Add(_string);
                RaisePropertyChangedEventSynchronous("Count");
            }
        }

        protected virtual void RaisePropertyChangedEventSynchronous(string propertyName)
        {
            try
            {
                if (this.PropertyChanged != null)
                {
                    Delegate[] delegates = this.PropertyChanged.GetInvocationList();
                    foreach (PropertyChangedEventHandler handler in delegates)
                    {
                        DispatcherObject dispatcherInvoker = handler.Target as DispatcherObject;
                        if (dispatcherInvoker != null)
                        {
                            dispatcherInvoker.Dispatcher.Invoke(DispatcherPriority.Normal, new Action(delegate
                            {
                                handler(this, new PropertyChangedEventArgs(propertyName));
                            }));
                        }
                        else
                        {
                            handler(this, new PropertyChangedEventArgs(propertyName));
                        }

                    }
                }
            }
            catch
            {
            }
        }
    }

    public abstract class Observable : INotifyPropertyChanged
    {
        [field: NonSerialized]
        public event PropertyChangedEventHandler PropertyChanged;
        
        #region Protected

        protected virtual void AfterPropertyChanged(string propertyName)
        {
            RaisePropertyChangedEvent(propertyName);
        }

        protected virtual void RaisePropertyChangedEvent(string propertyName)
        {            

            // TODO: consider caching event arg as indicated here:
            // http://joshsmithonwpf.wordpress.com/2007/08/29/a-base-class-which-implements-inotifypropertychanged/

            try
            {
                if (PropertyChanged != null)
                {
                    Delegate[] delegates = this.PropertyChanged.GetInvocationList();
                    foreach (PropertyChangedEventHandler handler in delegates)
                    {
                        DispatcherOperation operation = null;
                        try
                        {
                            DispatcherObject dispatcherInvoker = handler.Target as DispatcherObject;
                            if (dispatcherInvoker != null)
                            {
                                // changed from dispatcher.Invoke
                                operation = dispatcherInvoker.Dispatcher.BeginInvoke(DispatcherPriority.Normal, new Action(delegate
                                {
                                    handler(this, new PropertyChangedEventArgs(propertyName));
                                }));
                            }
                            else
                            {
                                handler(this, new PropertyChangedEventArgs(propertyName));
                            }
                        }
                        catch (Exception ex)
                        {
                           
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                
            }
        }

        protected virtual void RaisePropertyChangedEventSynchronous(string propertyName)
        {
         
            // TODO: consider caching event arg as indicated here:
            // http://joshsmithonwpf.wordpress.com/2007/08/29/a-base-class-which-implements-inotifypropertychanged/

            try
            {
                if (PropertyChanged != null)
                {
                    Delegate[] delegates = this.PropertyChanged.GetInvocationList();
                    foreach (PropertyChangedEventHandler handler in delegates)
                    {
                        DispatcherOperation operation = null;
                        try
                        {
                            DispatcherObject dispatcherInvoker = handler.Target as DispatcherObject;
                            if (dispatcherInvoker != null)
                            {
                                dispatcherInvoker.Dispatcher.Invoke(DispatcherPriority.Normal, new Action(delegate
                                {
                                    handler(this, new PropertyChangedEventArgs(propertyName));
                                }));
                            }
                            else
                            {
                                handler(this, new PropertyChangedEventArgs(propertyName));
                            }
                        }
                        catch (Exception ex)
                        {
                         
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                
            }
        }

        protected virtual void RaisePropertyChangedEvent(object sender, PropertyChangedEventArgs e)
        {
            if (PropertyChanged != null)
                PropertyChanged(sender, e);
        }


        //public static void RaisePropertyChangedEvent(INotifyPropertyChanged sender, string propertyName)
        //{
        //    Observable.VerifyProperty(sender, propertyName);

        //    // TODO: consider caching event arg as indicated here:
        //    // http://joshsmithonwpf.wordpress.com/2007/08/29/a-base-class-which-implements-inotifypropertychanged/

        //    if (sender.PropertyChanged != null)
        //    {
        //        Delegate[] delegates = sender.PropertyChanged.GetInvocationList();
        //        foreach (PropertyChangedEventHandler handler in delegates)
        //        {
        //            DispatcherObject dispatcherInvoker = handler.Target as DispatcherObject;
        //            if (dispatcherInvoker != null)
        //            {
        //                dispatcherInvoker.Dispatcher.Invoke(DispatcherPriority.Input, new Action(delegate
        //                {
        //                    handler(sender, new PropertyChangedEventArgs(propertyName));
        //                }));
        //            }
        //            else
        //            {
        //                handler(sender, new PropertyChangedEventArgs(propertyName));
        //            }
        //        }
        //    }
        //}



        #endregion

        #region Private Helpers

      
        #endregion
    }

    [Serializable]
    [DataContract(Namespace = "ems.gis")]
    public class MappingContextBase : Observable
    {
        string nameRandomizer = null;
        bool isNameSynchedToAlias = false;

        /// <summary>
        /// Set this to true for an auto unique name to be assigned to this instance.
        /// </summary>
        public bool CanAutoOverrideName { get; set; }

        /// <summary>
        /// Set to true of you do not want mapping context to modify name based on map name.
        /// </summary>
        public bool IsNameSynchedToAlias
        {
            get { return isNameSynchedToAlias; }
            set { isNameSynchedToAlias = value; }
        }

        public MappingContextBase()
        {
            //Name = Utility.Utilities.UniqueString;
            Name = this.GetHashCode().ToString();
            createNewLayer = true;
            IsVisible = true;
            IsReplacable = true;
            Opacity = 1;
            //CanAutoOverrideName = true;

        }

        int id = -1;
        public int Id
        {
            get
            {
                return id == -1 ? this.GetHashCode() : id;
            }
            set { id = value; }
        }

        bool createNewLayer;
        /// <summary>
        /// Set this to true if you want to create a new Data Manager, hence new layer for this
        /// request.  If falase, an existing DDM for this data set will be resolved, based on Name if one exist.
        /// If one does not exist, a new DDM will be created.        
        /// </summary>
        public bool CreateNewLayer
        {
            get { return createNewLayer; }
            set
            {
                if (createNewLayer != value)
                {
                    createNewLayer = value;
                    if (createNewLayer)
                        nameRandomizer = DateTime.Now.Ticks.ToString();

                    RaisePropertyChangedEvent("CreateNewLayer");
                }
            }
        }

     
        //[DataMember(Name="canSee")]
        public bool IsVisible { get; set; }


        string resourceURL;
        /// <summary>
        /// The resource URL for this layer's geodatabase.  If it is a file based gdb, this
        /// will be path to file.  If it is a webservice, such as WMS or WFS, this will be URL to 
        /// resource.  For WFS it will be workspace:table_name e.g ems:Cospas_Sarsat_POIs..
        /// </summary>
        [DataMember(Name = "rsrURL", EmitDefaultValue = false)]
        public string ResourceURL
        {
            get { return resourceURL; }
            set
            {
                resourceURL = value;
                RaisePropertyChangedEvent("ResourceURL");
            }
        }


        [DataMember(Name = "canCut")]
        public bool IsReplacable
        {
            get;
            set;
        }
      

        double opacity;
        [DataMember(Name = "hue")]
        public double Opacity
        {
            get { return opacity; }
            set
            {
                if (opacity != value)
                {
                    opacity = value;
                    RaisePropertyChangedEventSynchronous("Opacity");
                }
            }
        }

        string name;
        [DataMember(Name = "name", EmitDefaultValue = false)]
        public string Name
        {
            get
            {               
                return name;
            }
            set
            {
                name = value;
                RaisePropertyChangedEvent("Name");
            }
        }

     

        string alias;
        [DataMember(Name = "alias", EmitDefaultValue = false)]
        public string Alias
        {
            get { return alias; }
            set
            {
                alias = value;
                isNameSynchedToAlias = false;
                RaisePropertyChangedEvent("Alias");
            }
        }

        string description;
        [DataMember(Name = "descr", EmitDefaultValue = false)]
        public string Description
        {
            get { return description; }
            set
            {
                description = value;
                RaisePropertyChangedEvent("Description");
            }
        }

        string cacheLocation;
        [DataMember(Name = "cache", EmitDefaultValue = false)]
        public string Cache
        {
            get
            {
                if (string.IsNullOrEmpty(cacheLocation))
                    cacheLocation = StringHelper.GetValidFilename(name);
                return cacheLocation;
            }
            set
            {
                cacheLocation = value;
                RaisePropertyChangedEvent("Cache");
            }
        }          
    }

    [Serializable]
    [DataContract(Name = "fmRrc", Namespace = "ems.gis")]
    public class FileMapResource : MappingContextBase
    {
        /// <summary>
        /// Size of file in KB
        /// </summary>
        [DataMember(Name = "size", EmitDefaultValue = false)]
        public int FileSize { get; set; }
    }

    [Serializable]
    [DataContract(Name = "fmRrc", Namespace = "ems.gis")]
    public class ShapefileMapResource : FileMapResource
    {
        public ShapefileMapResource()
        {
            Initialize();
        }

        void Initialize()
        {

        }

        [OnDeserializing]
        void OnSerializing(StreamingContext c)
        {
            Initialize();
        }

        [DataMember(Name = "label", EmitDefaultValue = false)]
        public string LabelField { get; set; }

        [DataMember(Name = "labelOnly", EmitDefaultValue = false)]
        public bool LabelOnly { get; set; }
    }

    [Serializable]
    [CollectionDataContract(ItemName = "sf", Namespace = "ems.gis")]
    public class ShapefileResources : ObservableCollection<ShapefileMapResource>
    {

    }

    public class StringHelper
    {
        public static string GetValidFilename(string fileNameCandidate)
        {
            var invalids = System.IO.Path.GetInvalidFileNameChars();
            var _name = fileNameCandidate.ToCharArray();
            var validName = _name.Except(invalids).ToArray();
            return new string(validName);
        }

        public static int GetUniqueIdOfString(string inputString)
        {
            return RemoveAllSpacesFromStrng(inputString).GetHashCode();
        }

        public static string RemoveAllSpacesFromStrng(string inputString)
        {
            var regex = new Regex(@"\s+");
            return regex.Replace(inputString, string.Empty);

        }

        public static string RemoveAllSpacesFromEndofStrng(string inputString)
        {
            return inputString.TrimEnd(' ');
        }


        public static bool IsEmptyString(string inputString)
        {
            // match any non=white space character      
            var regex = new Regex(@"\S");
            return !regex.Match(inputString).Success;
        }

        public static string RemoveAllSpacesFromStartOfStrng(string inputString)
        {
            return inputString.TrimStart(' ');
        }

        public static string RemoveAllSpacesFromStartAndEndOfStrng(string inputString)
        {
            return inputString.TrimStart(' ').TrimEnd(' ');
        }

        /// <summary>
        /// Searches for part of string ignoring case.
        /// </summary>
        /// <param name="target"></param>
        /// <param name="partToFind"></param>
        /// <returns></returns>
        public static bool FindPartOfString(string target, string partToFind)
        {
            //bool found = false;
            //if (!string.IsNullOrEmpty(target) && partToFind != null)
            //    if (0 <= target.IndexOf(partToFind, StringComparison.InvariantCultureIgnoreCase))
            //        found = true;
            //return found;

            bool found = false;
            if (!string.IsNullOrEmpty(target) && partToFind != null)
                if (Regex.Match(target, partToFind, RegexOptions.IgnoreCase).Success)
                    found = true;
            return found;
        }      
    }    
}
